CONTAINS_ENCRYPTED_TAGS = "yaml contains encrypted tags"
CONTAINS_UNENCRYPTED_TAGS = "yaml contains unencrypted tags"


ANSIBLE_VAULT_ENCRYPTION_METHOD = 'ansible-vault'

VALID_ENCRYPTION_METHODS = [
    ANSIBLE_VAULT_ENCRYPTION_METHOD
]